;--------------------------------------------------------------------------------
;	Periferie DS1 - bojler, jimka
;	DS adresa 00000010
;
;	16F628A / 20MHz			
;	vse OFF, osc. HS, bez preruseni
;
;	02/2016 Milan rajer
;--------------------------------------------------------------------------------
;
;                             --------\/--------
;                        T2  -|RA2          RA1|-  T1
;                            -|RA3          RA0|-  BUS
;                            -|RA4         OSC1|-  \ Krystal 20 Mhz
;                     RESET  -|MCLR        OSC2|-  /
;                        0V  -|Vss          Vdd|-  +5V
;                      30cm  -|RB0          RB7|-  ICSP
;                      60cm  -|RB1          RB6|-  ICSP
;                      90cm  -|RB2          RB5|-  LED
;                     120cm  -|RB3          RB4|-  150cm
;                             ------------------
;
;--------------------------------------------------------------------------------

	LIST P=16F628A, R=DEC
	INCLUDE <P16F628A.INC>

;-------------------------------
;	Nastaveni pojistek
;-------------------------------

	org	0x2007		; adresa nastaveni pojistek pro PIC16F627 a PIC16F628A
	dw _BODEN_OFF & _CP_OFF & _PWRTE_OFF & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _HS_OSC 

;-------------------------------
;	Nastaveni dat v EEPROM
;-------------------------------

	org	0x2100		; tabulka v pameti EEPROM (128 bytu)
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
;		----------------------------------------------
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	de	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
;		----------------------------------------------

;--------------------------------------------------------------------------------
;	Nastaveni parametru
;-------------------------------
W	equ	0
F	equ	1
;--------------------------------------------------------------------------------
;	Definice registru, bitu a pinu
;-------------------------------
SYS	equ	0x20
;-----------------------------------
;|   |   |   |   |   |   |   | TMP |
;-----------------------------------
#define	TMP	SYS,0
;       ------------------------
TMP1	equ	0x21		;pomocny registr
TMP2	equ	0x22		;pomocny registr
TMP3	equ	0x23		;pomocny registr
TMP4	equ	0x24		;pomocny registr
TMP5	equ	0x25		;pomocny registr
;       ------------------------
CEK1	equ	0x26		;casova smycka
CEK2	equ	0x27		;casova smycka
CEK3	equ	0x28		;casova smycka
;       ------------------------
DAT	equ	0x29		;datovy registr
;       ------------------------
BCD	equ	0x2A		;cislo v BCD kodu
BCDD	equ	0x2B		;desitky v BCD kodu
BCDJ	equ	0x2C		;jednotky v BCD kodu
;-------------------------------
;	Pracovni registry DS18B20
;-------------------------------
DES	equ	0x30
;       ------------------------
LSB	equ	0x31
MSB	equ	0x32
TH	equ	0x33
TL	equ	0x34
CON	equ	0x35
RE01	equ	0x36
RE02	equ	0x37
RE03	equ	0x38
CRC	equ	0x39
;-------------------------------
;	Teplota T1
;-------------------------------
T1DE	equ	0x40		;T1 desitky
T1JE	equ	0x41		;T1 jednotky
T1DT	equ	0x42		;T1 desetiny
;-------------------------------
;	Teplota T2
;-------------------------------
T2DE	equ	0x43		;T2 desitky
T2JE	equ	0x44		;T2 jednotky
T2DT	equ	0x45		;T2 desetiny
;-------------------------------
;	Akumulator
;-------------------------------
JIM	equ	0x46		;jimka
;-------------------------------
;	Sbernice
;-------------------------------
#define	BUS	PORTA,0		;sbernice
#define	BUS_T	TRISA,0		;tris registr sbernice
;-------------------------------
;	Periferie
;-------------------------------
#define	J30	PORTB,0		;jimka 30cm
#define	J60	PORTB,1		;jimka 60cm
#define	J90	PORTB,2		;jimka 90cm
#define	J120	PORTB,3		;jimka 120cm
#define	J150	PORTB,4		;jimka 150cm

#define	T1	PORTA,1		;teplotni cidlo 1
#define	T1_T	TRISA,1		;tris registr cidla 1

#define	T2	PORTA,2		;teplotni cidlo 2
#define	T2_T	TRISA,2		;tris registr cidla 2

#define	LED	PORTB,5		;LED dioda (nyni neaktivni)
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;	Zacatek behu programu
;-------------------------------
	org	0x0000
	goto	initial
;--------------------------------------------------------------------------------
;	Knihovny
;-------------------------------
	#include	<piclib_bus_v2.inc>		;komunikace po sbernici
	#include	<piclib_2xDS18B20_v3.inc>	;komunikace s 2x DS18B20
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;	Inicializace po zapnuti
;-------------------------------
;-------------------------------
;	nastaveni portu
;-------------------------------
initial	bcf	STATUS,RP0	;banka 0
	bcf	STATUS,RP1
;       ------------------------
	clrf	PORTA		;vymazani vsech portu
	clrf	PORTB
;
	movlw	b'00000111'	; digitalni vstupy na PORTu A (PIC16F627 a 628)
	movwf	CMCON 
;       ------------------------
	bsf	STATUS,RP0	;banka 1
	bcf	STATUS,RP1
;       ------------------------
	movlw	b'11111'
	movwf	TRISA		;port A vstupni/vystupni

	movlw	b'11010100'	; PULL-UPy OFF, preddelic=1:32 pripojen k TMR0
	movwf	OPTION_REG	; f:4  :32  :256 = 122,1Hz (8,192ms)

	movlw	b'11111111'
	movwf	TRISB		;port B vstupni/vystupni
;       ------------------------
	bcf	STATUS,RP0	;banka 0
	bcf	STATUS,RP1
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;
;	*****	*****	*****	****	*****
;	*	  *	*   *	*   *	  *
;	*****	  *	***** 	****	  *
;	    *	  *	*   *	*  *	  *
;	*****	  *	*   *	*   *	  *
;
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
start	clrf	T1DE		;vynuluj merene hodnoty
	clrf	T1JE		;
	clrf	T1DT		;
;       ------------------------
	clrf	T2DE		;
	clrf	T2JE		;
	clrf	T2DT		;
;       ------------------------
	clrf	JIM		;
;-------------------------------
;	Mereni teplot T1 a T2
;-------------------------------
mereni	call	t1_mer		;zmer T1
	call	t2_mer		;zmer T2
;       ------------------------
	call	c0s8		;pockej 0,8s na dokonceni procesu (>0,7s)
;       ------------------------
	call	t1_cti		;cti T1
	call	t2_cti		;cti T2
;-------------------------------
;	Nacte a vyhodnoti stav jimky
;-------------------------------
	btfss	J150		;je hladina nad 150cm?
	goto	J_150
;	------------------------
	btfss	J120		;je hladina nad 120cm?
	goto	J_120
;	------------------------
	btfss	J90		;je hladina nad 90cm?
	goto	J_90		
;	------------------------
	btfss	J60		;je hladina nad 60cm?
	goto	J_60		
;	------------------------
	btfss	J30		;je hladina nad 30cm?
	goto	J_30		
;-------------------------------
	clrf	JIM		;nastav priznak
;	------------------------
	goto	rutina
;-------------------------------
J_30	movlw	b'00000001'	;30cm
	movwf	JIM
;	------------------------
	goto	rutina
;-------------------------------
J_60	movlw	b'00000010'	;60cm
	movwf	JIM
;	------------------------
	btfsc	J30		;chybovy priznak
	bsf	JIM,7
;	------------------------
	goto	rutina
;-------------------------------
J_90	movlw	b'00000100'	;nastav priznak
	movwf	JIM
;	------------------------
	btfsc	J30		;chybovy priznak
	bsf	JIM,7
	btfsc	J60
	bsf	JIM,7
;	------------------------
	goto	rutina
;-------------------------------
J_120	movlw	b'00001000'	;nastav priznak
	movwf	JIM
;	------------------------
	btfsc	J30		;chybovy priznak
	bsf	JIM,7
	btfsc	J60
	bsf	JIM,7
	btfsc	J90
	bsf	JIM,7
;	------------------------
	goto	rutina
;-------------------------------
J_150	movlw	b'00010000'	;nastav priznak
	movwf	JIM
;	------------------------
	btfsc	J30		;chybovy priznak
	bsf	JIM,7
	btfsc	J60
	bsf	JIM,7
	btfsc	J90
	bsf	JIM,7
	btfsc	J120
	bsf	JIM,7
;	------------------------
	goto	rutina
;-------------------------------
;	Neustale probihajici detekce reset impulsu
;-------------------------------
rutina	btfsc	BUS		;cekej na log 0
	goto	rutina
;-------------------------------
	movlw	.150		;test delky impulsu log 0 po dobu 120us
        movwf	CEK1
;       ------------------------
	btfsc	BUS		;pokud je impuls kratsi, vrat se na rutinu
	goto	rutina
;       ------------------------
	decfsz	CEK1,1
        goto	$-3		;..jinak pokracuj
;-------------------------------
	movlw	.11		;cekej max 1200us na konec reset impulsu (log 1)
        movwf	CEK2
rut12   movlw	.135
        movwf	CEK1
;       ------------------------
rut11	btfsc	BUS		;pokud behem testu prejde BUS do log 1..
	goto	nac_adr		;..pokracuj v prijmu
;       ------------------------
	decfsz	CEK1,1
        goto	rut11
        decfsz	CEK2,1
        goto	rut12		;pokud se log 1 neobevi, vrat se na rutinu
	goto	rutina
;-------------------------------
;	Prijem adresy periferie (00000010)
;-------------------------------
nac_adr	call	bus_dat		;precte byte ze sbernice
;       ------------------------
	movlw	b'00000010'	;je tazano toto zarizeni?
	subwf	DAT,W
	btfss	STATUS,Z	;pokud ne, vrat se na rutinu, jinak pokracuj
	goto	rutina
;-------------------------------
;	Vyslani dat na sbernici
;-------------------------------
vysli	call	bus_rw		;prodleva pred zacatkem vysilani
;       ------------------------
	movf	T1DE,W		;T1
	call	w_bus
	movf	T1JE,W
	call	w_bus
	movf	T1DT,W
	call	w_bus
;       ------------------------
	movf	T2DE,W		;T2
	call	w_bus
	movf	T2JE,W
	call	w_bus
	movf	T2DT,W
	call	w_bus
;       ------------------------
	movf	JIM,W		;JIM
	call	w_bus
;       ------------------------
	movlw	b'01010101'	;kontrolni byte
	call	w_bus
;       ------------------------
	goto	mereni		;po vyslani dat se vrat a proved nove mereni
;-------------------------------------------------------------------------------
;cekaci smycky
;-------------------------------------------------------------------------------
c0s8	movlw	.69  		;(30)
        movwf	CEK3
        movlw	.91		;(255)
        movwf	CEK2
        movlw	.211		;(255)
        movwf	CEK1
;       ------------------------
        decfsz	CEK1,F
        goto	$-1
        decfsz	CEK2,F
        goto	$-5
        decfsz	CEK3,F
        goto	$-9
;       ------------------------
	return
;-------------------------------------------------------------------------------
	end