;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;
;								---------
;	Knihovna pro ovladani dvojice teplotnich cidel DS18B20	|	|
;								|DS18B20|
;	Procesor:	PIC16					|	|
;	Rychlost:	20MHz					---------
;	Umisteni:	stranka 0				  | | |
;								  | | |
;	02/2016 Milan rajer					  | | |
;								GND D +5V
;
;-------------------------------------------------------------------------------
;
;	call	t1_mer		<- zmeri teplotu T1
;	call	t2_mer		<- zmeri teplotu T2
;
;
;	call	t1_cti		<- precte data z cidla T1
;	------------------------
;	T1DE - desitky		<- vraci
;	T1JE - jednotky
;	T1DT - desetiny
;
;
;	call	t2_cti		<- precte data z cidla T2
;	------------------------
;	T2DE - desitky		<- vraci
;	T2JE - jednotky
;	T2DT - desetiny
;
;-------------------------------------------------------------------------------
;
;	Pozadavky - registry:
;	------------------------
;	DAT - pracovni registr
;	DES - pracovni registr
;
;	T1DE - desitky
;	T1JE - jednotky
;	T1DT - desetiny
;
;	T2DE - desitky
;	T2JE - jednotky
;	T2DT - desetiny
;
;	LSB - parametry cidla
;	MSB
;	TH
;	TL
;	CON
;	RE01
;	RE02
;	RE03
;	CRC
;
;	TMP1 - pomocny registr
;
;	CEK1 - registry casovych smycek
;	CEK2
;
;
;	Pozadavky - bity:
;	------------------------
;	TMP - pomocny bit
;
;
;	Pozadavky - piny:
;	------------------------
;	T1 - pin sbernice 1-Wire		(#define	T1	PORTA,1)
;	T1_T - pozice pinu T1 v registru TRIS	(#define	T1_T	TRISA,1)
;
;	T2 - pin sbernice 1-Wire		(#define	T2	PORTA,2)
;	T2_T - pozice pinu T2 v registru TRIS	(#define	T2_T	TRISA,2)
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;	Vysle prikaz ke zmereni T1
;--------------------------------------------------------------------------------
t1_mer	call	res_t1		;vysli reset impuls na sbernici T1
	call	t_cm5		;cekej 500us
;	------------------------
	call	sk_rom		;vysli skip-rom do T1
	call	dat_t1		;
	call	t_cm5		;cekej 500us
;	------------------------
	call	co_tep		;vysli convet-T do T1
	call	dat_t1		;
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Vysle prikaz ke zmereni T2
;--------------------------------------------------------------------------------
t2_mer	call	res_t2		;vysli reset impuls na sbernici T2
	call	t_cm5		;cekej 500us
;	------------------------
	call	sk_rom		;vysli skip-rom do T2
	call	dat_t2		;
	call	t_cm5		;cekej 500us
;	------------------------
	call	co_tep		;vysli convet-T do T2
	call	dat_t2		;
;	------------------------
	return
;--------------------------------------------------------------------------------	
;	Cteni scratchpad a prevod teploty T1 do dec, vraci T1DE, T1JE, T1DT
;--------------------------------------------------------------------------------
t1_cti	call	res_t1		;vysli reset impuls na sbernici T1
	call	t_cm5		;cekej 500us
;	------------------------
	call	sk_rom		;vysli skip-rom do T1
	call	dat_t1		;
	call	t_cm5		;cekej 500us
;	------------------------
	call	re_scr		;vysli read-scretchpad do T1
	call	dat_t1		;
	call	t_cm5		;cekej 500us
;	------------------------
	call	t1_dat		;LSB
	movf	DAT,0		;nacti obsah registru DAT
	movwf	LSB		;..a vloz ho do..
	call	t1_dat		;MSB
	movf	DAT,0		;nacti obsah registru DAT
	movwf	MSB		;..a vloz ho do..
	call	t1_dat		;TH
	movf	DAT,0		;nacti obsah registru DAT
	movwf	TH		;..a vloz ho do..
	call	t1_dat		;TL
	movf	DAT,0		;nacti obsah registru DAT
	movwf	TL		;..a vloz ho do..
	call	t1_dat		;CON
	movf	DAT,0		;nacti obsah registru DAT
	movwf	CON		;..a vloz ho do..
	call	t1_dat		;RE01
	movf	DAT,0		;nacti obsah registru DAT
	movwf	RE01		;..a vloz ho do..
	call	t1_dat		;RE02
	movf	DAT,0		;nacti obsah registru DAT
	movwf	RE02		;..a vloz ho do..
	call	t1_dat		;RE03
	movf	DAT,0		;nacti obsah registru DAT
	movwf	RE03		;..a vloz ho do..
	call	t1_dat		;CRC
	movf	DAT,0		;nacti obsah registru DAT
	movwf	CRC		;..a vloz ho do..
;	------------------------
	call	preved
;	------------------------
	movf	MSB,0		;uloz prectene hodnoty
	movwf	T1DE
	movf	LSB,0
	movwf	T1JE
	movf	DES,0
	movwf	T1DT
;	------------------------
	return
;--------------------------------------------------------------------------------	
;	Cteni scratchpad a prevod teploty T2 do dec, vraci T2DE, T2JE, T2DT
;--------------------------------------------------------------------------------
t2_cti	call	res_t2		;vysli reset impuls na sbernici T2
	call	t_cm5		;cekej 500us
;	------------------------
	call	sk_rom		;vysli skip-rom do T2
	call	dat_t2		;
	call	t_cm5		;cekej 500us
;	------------------------
	call	re_scr		;vysli read-scretchpad do T2
	call	dat_t2		;
	call	t_cm5		;cekej 500us
;	------------------------
	call	t2_dat		;LSB
	movf	DAT,0		;nacti obsah registru DAT
	movwf	LSB		;..a vloz ho do..
	call	t2_dat		;MSB
	movf	DAT,0		;nacti obsah registru DAT
	movwf	MSB		;..a vloz ho do..
	call	t2_dat		;TH
	movf	DAT,0		;nacti obsah registru DAT
	movwf	TH		;..a vloz ho do..
	call	t2_dat		;TL
	movf	DAT,0		;nacti obsah registru DAT
	movwf	TL		;..a vloz ho do..
	call	t2_dat		;CON
	movf	DAT,0		;nacti obsah registru DAT
	movwf	CON		;..a vloz ho do..
	call	t2_dat		;RE01
	movf	DAT,0		;nacti obsah registru DAT
	movwf	RE01		;..a vloz ho do..
	call	t2_dat		;RE02
	movf	DAT,0		;nacti obsah registru DAT
	movwf	RE02		;..a vloz ho do..
	call	t2_dat		;RE03
	movf	DAT,0		;nacti obsah registru DAT
	movwf	RE03		;..a vloz ho do..
	call	t2_dat		;CRC
	movf	DAT,0		;nacti obsah registru DAT
	movwf	CRC		;..a vloz ho do..
;	------------------------
	call	preved
;	------------------------
	movf	MSB,0		;uloz prectene hodnoty
	movwf	T2DE
	movf	LSB,0
	movwf	T2JE
	movf	DES,0
	movwf	T2DT
;	------------------------
	return
;--------------------------------------------------------------------------------	
;	
;	Prevod teploty do BCD
;
;	Pokud je cteni v poradku, vraci MSB (desitky), LSB (jednotky) a DES (desetiny).
;	V pripade nezdareneho cteni vraci 0,0C.
;
;--------------------------------------------------------------------------------	
preved	movf	RE03,0		;zkontroluj cteni z cidla (RE03 musi byt 10h)
	sublw	0x10
	btfss	STATUS,Z	;pokud RE0 neni 10h
	goto	chybaT		;..jdi na chybu
;-------------------------------
	clrf	TMP1
	bcf	STATUS,C	;vymaz carry a priprav LSB + MSB
	rrf	MSB
	rrf	LSB
	rrf	TMP1
	rrf	MSB
	rrf	LSB
	rrf	TMP1
	rrf	MSB
	rrf	LSB
	rrf	TMP1
	rrf	MSB
	rrf	LSB
	rrf	TMP1
;	------------------------
BinBCD	clrf	MSB		;prevod celych stupnu
gtenth	movlw	.10
	subwf	LSB,0
	btfss	STATUS,C
	goto	over
	movwf	LSB
	incf	MSB,1
	goto	gtenth
over
;-------------------------------
BinDES	swapf	TMP1,1		;prevod desetin
;
	movlw	0x00
;
	decf	TMP1,1		;0,0625
	btfsc	STATUS,Z
	movlw	.1		;0,1
	decf	TMP1,1		;0,125
	btfsc	STATUS,Z
	movlw	.1		;0,1
	decf	TMP1,1		;0,1875
	btfsc	STATUS,Z
	movlw	.2		;0,2
	decf	TMP1,1		;0,25
	btfsc	STATUS,Z
	movlw	.3		;0,3
	decf	TMP1,1		;0,3125
	btfsc	STATUS,Z
	movlw	.3		;0,3
	decf	TMP1,1		;0,375
	btfsc	STATUS,Z
	movlw	.4		;0,4
	decf	TMP1,1		;0,4375
	btfsc	STATUS,Z
	movlw	.4		;0,4
	decf	TMP1,1		;0,5
	btfsc	STATUS,Z
	movlw	.5		;0,5
	decf	TMP1,1		;0,5625
	btfsc	STATUS,Z
	movlw	.6		;0,6
	decf	TMP1,1		;0,625
	btfsc	STATUS,Z
	movlw	.6		;0,6
	decf	TMP1,1		;0,6875
	btfsc	STATUS,Z
	movlw	.7		;0,7
	decf	TMP1,1		;0,75
	btfsc	STATUS,Z
	movlw	.8		;08
	decf	TMP1,1		;0,8125
	btfsc	STATUS,Z
	movlw	.8		;0,8
	decf	TMP1,1		;0,875
	btfsc	STATUS,Z
	movlw	.9		;0,9
	decf	TMP1,1		;0,9375
	btfsc	STATUS,Z
	movlw	.9		;0,9
;	------------------------
	movwf	DES
;	------------------------
	return
;-------------------------------
chybaT	clrf	MSB		;vymaz vsechny udaje
	clrf	LSB
	clrf	DES
;	------------------------
	return
;-------------------------------------------------------------------------------
;	Nastaveni pinu T1
;-------------------------------------------------------------------------------
t1_in	bsf	STATUS,RP0	;banka 1
	bcf	STATUS,RP1
	bsf	T1_T		;T1 vstupni
	bcf	STATUS,RP0	;banka 0
;	------------------------
	return
;-------------------------------
t1_out	bcf	T1
	bsf	STATUS,RP0	;banka 1
	bcf	STATUS,RP1
	bcf	T1_T		;T1 vystupni
	bcf	STATUS,RP0	;banka 0
;	------------------------
	return
;-------------------------------------------------------------------------------
;	Nastaveni pinu T2
;-------------------------------------------------------------------------------
t2_in	bsf	STATUS,RP0	;banka 1
	bcf	STATUS,RP1
	bsf	T2_T		;T2 vstupni
	bcf	STATUS,RP0	;banka 0
;	------------------------
	return
;-------------------------------
t2_out	bcf	T2
	bsf	STATUS,RP0	;banka 1
	bcf	STATUS,RP1
	bcf	T2_T		;T2 vystupni
	bcf	STATUS,RP0	;banka 0
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Reset impuls T1
;--------------------------------------------------------------------------------
res_t1	call	t1_out		;T1 do log 0 na 500us
	call	t_cm5		;
	call	t1_in		;T1 do log 1 na 80us
	call	t_c80u		;
;       ------------------------
	return
;--------------------------------------------------------------------------------
;	Reset impuls T2
;--------------------------------------------------------------------------------
res_t2	call	t2_out		;T2 do log 0 na 500us
	call	t_cm5		;
	call	t2_in		;T2 do log 1 na 80us
	call	t_c80u		;
;       ------------------------
	return
;--------------------------------------------------------------------------------
;	Cteni byte z cidla T1, vraci DAT
;--------------------------------------------------------------------------------
t1_dat	clrf	DAT		;vynuluj registr DAT
;	------------------------
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,0		;nastav DAT
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,1		;nastav DAT
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,2		;nastav DAT
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,3		;nastav DAT
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,4		;nastav DAT
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,5		;nastav DAT
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,6		;nastav DAT
	call	t1_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,7		;nastav DAT
;	------------------------
	return
;-------------------------------
;	Cteni bit z cidla T1, vraci TMP
;-------------------------------
t1_bit	bcf	TMP		;vynuluj TMP
;	------------------------
	call	t1_out		;T1 do log 0 na 5us
	call	t_c5u		;
	call	t1_in		;uvolneni T1
;	------------------------
	call	t_c10u		;cekej 10us
;	------------------------
	btfsc	T1		;kdyz je T1 v log 1..
	bsf	TMP		;nastav TMP
;	------------------------
	call	t_c80u		;cekej 80us
;	------------------------
	return
;--------------------------------------------------------------------------------
;	Cteni byte z cidla T2, vraci DAT
;--------------------------------------------------------------------------------
t2_dat	clrf	DAT		;vynuluj registr DAT
;	------------------------
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,0		;nastav DAT
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,1		;nastav DAT
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,2		;nastav DAT
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,3		;nastav DAT
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,4		;nastav DAT
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,5		;nastav DAT
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,6		;nastav DAT
	call	t2_bit		;vyzadej si bit
	btfsc	TMP		;kdyz je TMP v 1..
	bsf	DAT,7		;nastav DAT
;	------------------------
	return
;-------------------------------
;	Cteni bit z cidla T2, vraci TMP
;-------------------------------
t2_bit	bcf	TMP		;vynuluj TMP
;	------------------------
	call	t2_out		;T2 do log 0 na 5us
	call	t_c5u		;
	call	t2_in		;uvolneni T2
;	------------------------
	call	t_c10u		;cekej 10us
;	------------------------
	btfsc	T2		;kdyz je T2 v log 1..
	bsf	TMP		;nastav TMP
;	------------------------
	call	t_c80u		;cekej 80us
;	------------------------
	return
;--------------------------------------------------------------------------------
;	vyslani datoveho byte DAT do teplomeru T1
;--------------------------------------------------------------------------------
dat_t1	btfss	T1		;Ceka na uvolneni sbernice
	goto	$-1
;       ------------------------
	btfsc	DAT,0		;bit 7
	call	log1_t1
	btfss	DAT,0
	call	log0_t1
;       ------------------------
	btfsc	DAT,1		;bit 6
	call	log1_t1
	btfss	DAT,1
	call	log0_t1
;       ------------------------
	btfsc	DAT,2		;bit 5
	call	log1_t1
	btfss	DAT,2
	call	log0_t1
;       ------------------------
	btfsc	DAT,3		;bit 4
	call	log1_t1
	btfss	DAT,3
	call	log0_t1
;       ------------------------
	btfsc	DAT,4		;bit 3
	call	log1_t1
	btfss	DAT,4
	call	log0_t1
;       ------------------------
	btfsc	DAT,5		;bit 2
	call	log1_t1
	btfss	DAT,5
	call	log0_t1
;       ------------------------
	btfsc	DAT,6		;bit 1
	call	log1_t1
	btfss	DAT,6
	call	log0_t1
;       ------------------------
	btfsc	DAT,7		;bit 0
	call	log1_t1
	btfss	DAT,7
	call	log0_t1
;       ------------------------
	return
;-------------------------------
;	vysle bit log 0
;-------------------------------
log0_t1	call	t1_out		;TEP vystupni (log 0) na 80us
	call	t_c80u
;       ------------------------
	call	t1_in		;TEP vstupni (log 1) na 10us
	call	t_c10u
;       ------------------------
	return
;-------------------------------
;	vysle bit log 1
;-------------------------------
log1_t1	call	t1_out		;TEP vystupni (log 0) na 10us
	call	t_c10u
;       ------------------------
	call	t1_in		;TEP vstupni (log 1) na 80us
	call	t_c80u
;       ------------------------
	return
;--------------------------------------------------------------------------------
;	vyslani datoveho byte DAT do teplomeru T2
;--------------------------------------------------------------------------------
dat_t2	btfss	T2		;Ceka na uvolneni sbernice
	goto	$-1
;       ------------------------
	btfsc	DAT,0		;bit 7
	call	log1_t2
	btfss	DAT,0
	call	log0_t2
;       ------------------------
	btfsc	DAT,1		;bit 6
	call	log1_t2
	btfss	DAT,1
	call	log0_t2
;       ------------------------
	btfsc	DAT,2		;bit 5
	call	log1_t2
	btfss	DAT,2
	call	log0_t2
;       ------------------------
	btfsc	DAT,3		;bit 4
	call	log1_t2
	btfss	DAT,3
	call	log0_t2
;       ------------------------
	btfsc	DAT,4		;bit 3
	call	log1_t2
	btfss	DAT,4
	call	log0_t2
;       ------------------------
	btfsc	DAT,5		;bit 2
	call	log1_t2
	btfss	DAT,5
	call	log0_t2
;       ------------------------
	btfsc	DAT,6		;bit 1
	call	log1_t2
	btfss	DAT,6
	call	log0_t2
;       ------------------------
	btfsc	DAT,7		;bit 0
	call	log1_t2
	btfss	DAT,7
	call	log0_t2
;       ------------------------
	return
;-------------------------------
;	vysle bit log 0
;-------------------------------
log0_t2	call	t2_out		;TEP vystupni (log 0) na 80us
	call	t_c80u
;       ------------------------
	call	t2_in		;TEP vstupni (log 1) na 10us
	call	t_c10u
;       ------------------------
	return
;-------------------------------
;	vysle bit log 1
;-------------------------------
log1_t2	call	t2_out		;TEP vystupni (log 0) na 10us
	call	t_c10u
;       ------------------------
	call	t2_in		;TEP vstupni (log 1) na 80us
	call	t_c80u
;       ------------------------
	return
;--------------------------------------------------------------------------------
;	Knihovna prikazu cidla
;--------------------------------------------------------------------------------
re_rom	movlw	0x33		;READ ROM
	movwf	DAT
	return
;-------------------------------
ma_rom	movlw	0x55		;MATCH ROM
	movwf	DAT
	return
;-------------------------------
sk_rom	movlw	0xCC		;SKIP ROM
	movwf	DAT
	return
;-------------------------------
se_rom	movlw	0xF0		;SEARCH ROM
	movwf	DAT
	return
;-------------------------------
al_sea	movlw	0xEC		;ALARM SEARCH
	movwf	DAT
	return
;-------------------------------
wr_scr	movlw	0x4E		;WRITE SCRATCHPAD
	movwf	DAT
	return
;-------------------------------
re_scr	movlw	0xBE		;READ SCRATCHPAD
	movwf	DAT
	return
;-------------------------------
co_scr	movlw	0x48		;COPY SCRATCHPAD
	movwf	DAT
	return
;-------------------------------
co_tep	movlw	0x44		;CONVERT T
	movwf	DAT
	return
;-------------------------------
re_e2	movlw	0xB8		;RECALL E2
	movwf	DAT
	return
;-------------------------------
re_pow	movlw	0xB4		;READ POWER SUPPLY
	movwf	DAT
	return
;--------------------------------------------------------------------------------
;	Cekaci smycky
;--------------------------------------------------------------------------------
t_c5u	movlw	.8		;cekej 5us
        movwf	CEK1
	decfsz	CEK1,1
        goto	$-1
	return
;-------------------------------
t_c10u	movlw	.15		;cekej 10us
        movwf	CEK1
	decfsz	CEK1,1
        goto	$-1
	return
;-------------------------------
t_c80u	movlw	.135		;cekej 80us
        movwf	CEK1
	decfsz	CEK1,1
        goto	$-1
	return
;-------------------------------
t_cm5	movlw	.51		;cekej 500us
        movwf	CEK2
        movlw	.15
        movwf	CEK1
	decfsz	CEK1,1
        goto	$-1
        decfsz	CEK2,1
        goto	$-5
	return
;--------------------------------------------------------------------------------